function [sys,x0,str,ts] = fcc_ekf(t,x,u,flag)
% fcc_ekf is an S-function which implement extended kalman filter for fcc
% Rf:
% [1] Gauthier-Kupka
% [2] Busvelle-Gauthier
global theta0 lambda
theta0=10;
theta1=5;
TsurN=3;
lambda=log((theta0-1)/(theta1-1))/TsurN/3600;
%
% Dispatch the flag.
%
dperiod=1;
doffset=0;
%
switch flag,
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes(dperiod,doffset); % Initialization
  case 1
    sys = mdlDerivatives(t,x,u); % Calculate derivatives
  case 2
    sys = mdlUpdate(t,x,u,dperiod,doffset); % Update states
  case 3
    sys = mdlOutputs(t,x,u); % Calculate outputs  
  case { 2, 4, 9 } % Unused flags
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); % Error handling
end
% End of csfunc.
%==============================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the 
% S-function.
%==============================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(dperiod,doffset)
global theta0 lambda
%
% Call simsizes for a sizes structure, fill it in and convert it 
% to a sizes array.
%
sizes = simsizes;
sizes.NumContStates  = 21;
sizes.NumDiscStates  = 21;
sizes.NumOutputs     = 10;
sizes.NumInputs      = 5;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 2;
sys = simsizes(sizes);
%
% Initialize the initial conditions.
%
% Auparavant, une simu  theta constant a t ralise et
% l'tat final ainsi que la matrice de covariance finale
% ont t sauvs dans xini.mat, vecteur 20 de 20 composantes.
%P0=zeros(5,5);
%x0=[775;943;0.3;1.2;0];
%x0=abs(x0.*(1+0.01*randn(size(x0))));
%x0=[x0;SymReshape(P0);theta0];
load xini.mat
x0=[xini;theta0];
x0=[x0;zeros(size(x0))];
%
% str is an empty matrix.
%
str = [];
%
% Initialize the array of sample times; in this example the sample 
% time is continuous, so set ts to 0 and its offset to 0.
%
ts = [0,0;dperiod,doffset];
% End of mdlInitializeSizes.
%
%==============================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%==============================================================
function sys = mdlDerivatives(t,x,u)
global theta0 lambda
% NAMES / DESCRIPTION / UNITS
%   R     Ideal gas constant J/mol.K
%   H     Holdup kg
%   R     Rate kg/s
%   S     Specific heat J/kg.K
%   P     Pressure kPa
%   T     Temperature K
%   D     Heat J/kg
%   A     Activation energy J/mol
%   F     factor, without unit
%   k     empirical constant rates /kPa.s
R = 8.314;
%
% Reactor
% -------
Hra = 1.85e4;
Pra = 211.7;
DHcr = 4.65e5;
% feed
Rtf = 41;
Ttf = 492.8;
Stf = 3140;
% Reaction 1
Acr = 6.28e4;
kcr = 8.31e-2;
% Reaction 2
Acc = 4.18e4;
kcc = 2.66e-4;
% REGROUPEMENT (CHGT DE NOTATION)
Acr=2*Acr-Acc;
kcr=kcr^2/kcc;
% Regenerator
% -----------
Hrg = 1.53e5;
Prg = 254.4;
DHfv = 1.74e5;
DHrg = 3.02e7;
% Air
Rai = 26;
Tai = 394;
Sai = 1130;
% Catalyst
Rc  = 290;
Sc  = 1047;
% Reaction (02)
Aor = 1.47e5;
%
x=x(1:21)+x(22:42);
%
Tra = x(1);
Trg = x(2);
Crc = x(3);
Csc = x(4);
Fcf = x(5);

P=SymReshape(x(6:20));

Rai = u(1);
dRai = u(2);
Tra_mes = u(3);
Trg_mes = u(4);

theta = x(21);

if Crc<=1e-3,
    Crc=1e-3;
    disp(['Crc saturation at t = ',num2str(t)]),
end,

Rcr = sqrt(kcr*Rc*Pra*Hra)/(10*spower(Crc,0.12,1e-3,t))*exp( -Acr/(2*R*Tra) );
Ctf = Rcr / ( Rcr + Rtf);
dTra = ( Sc*Rc*(Trg-Tra) + Stf*Rtf*(Ttf-Tra) - Rtf*DHfv - Rtf*Ctf*DHcr ) / (Sc*Hra);

Kor = 1.16e-5*exp( Aor/R*( 1/866.7 - 1/Trg ) );
Kod = 6.34e-9*Rai^2;
Ofg = 21*exp( - (Prg*Hrg/Rai) / ( 1/Kod + 1/(Kor*Crc) ) );
Rcb = Rai*(21-Ofg)/242;
dTrg = ( Sc*Rc*(Tra-Trg) + Sai*Rai*(Tai-Trg) + Rcb*DHrg ) / (Sc*Hrg);

dCrc = ( Rc*(Csc-Crc) - 100*Rcb )/Hrg;

Rcc = sqrt(kcc*Rc*Pra*Hra)/(10*spower(Crc,0.03,1e-3,t))*exp( -Acc/(2*R*Tra) );
Rad = Fcf*Rtf;
Rcf = Rcc + Rad;
dCsc = ( Rc*(Crc-Csc) + 100*Rcf )/Hra;

dFcf = 0;

C=[eye(2),zeros(2,3)];
DeltaInv=diag([1,1,theta,theta^2,theta^3]);
Q=diag([800,1000,0.3,1.2,1e-7])/1e14;
Q=theta^2*DeltaInv*Q*DeltaInv;
R1=eye(2);
% The following line has been tested and works well
%Q=theta*diag([1,1,0.1,0.1,0.001,0.5,0.3,0.1])/100000; % ATTENTION: PROVISOIRE
% Now, another solution more theoretically justified
%
% Jacobian matrix of our change of coordinates:
xi=zeros(1,5);
xi(1)=Tra;
xi(2)=1/(Rcr+Rtf);
xi(3)=xi(2)^2*Rcr/Crc*Rc*Csc;
xi(4)=xi(3)*Fcf/Csc;
xi(5)=Trg;
%
dxi=ones(5);
A_=(-Acr-2*Acc)/(2*R);
B_=sqrt(kcr*Pra*Hra*Rc/100);
C_=-Acc/(2*R);
a_=0.12;
b_=0.03;
dxi(2,1)=-1/a_*Crc^(1+a_)/B_*(1/xi(2)-Rtf)*exp(-A_/xi(1))*A_/xi(1)^2;
dxi(2,2)=+1/a_*Crc^(1+a_)/B_*exp(-A_/xi(1))/xi(2)^2;
dxi(3,1)=Csc*((b_+1)/Crc*dxi(2,1)+C_/xi(1)^2);
dxi(3,2)=Csc*((b_+1)/Crc*dxi(2,2)-2/xi(2));
dxi(3,3)=Csc/xi(3);
dxi(4,1)=Fcf*((b_+1)/Crc*dxi(2,1)+C_/xi(1)^2);
dxi(4,2)=Fcf*((b_+1)/Crc*dxi(2,2)-2/xi(2));
dxi(4,4)=dxi(3,3);
%
%q=dxi*Q*dxi';
R1=eye(2);
%
%

xdot = [dTra;dTrg;dCrc;dCsc;dFcf] + P*C'*R1*[Tra_mes-Tra;Trg_mes-Trg];
%
% Jacobien
% --------
A=zeros(5,5);
% Equation Tra
dRcr_dCrc = -0.12*Rcr/Crc;
dRcr_dTra = (Acr-Acc/2)/(R*Tra^2)*Rcr;
dCtf_dRcr = Rtf/(Rcr+Rtf)^2;
dCtf_dCrc = dCtf_dRcr * dRcr_dCrc;
dCtf_dTra = dCtf_dRcr * dRcr_dTra;
A(1,1) = ( -Sc*Rc -Stf*Rtf -Rtf*DHcr*dCtf_dTra) / (Sc*Hra);
A(1,2) = Rc/Hra;
A(1,3) = ( -Rtf*DHcr*dCtf_dCrc) / (Sc*Hra);
% Equation Trg
A(2,1) = Rc/Hrg;
A(2,2) = ( -Sc*Rc -Sai*Rai) / (Sc*Hrg);
% Equation Crc
A(3,3) = -Rc/Hrg;
A(3,4) = Rc/Hrg;
% Equation Csc
dRcf_dTra = Acc/(2*R*Tra^2)*Rcf;
dRcf_dCrc = -0.03/Crc*Rcf;
dRcf_dFcf = Rtf;
A(4,1) = 100*dRcf_dTra/Hra;
A(4,3) = (Rc+100*dRcf_dCrc)/Hra;
A(4,4) = -Rc/Hra;
A(4,5) = 100*dRcf_dFcf/Hra;

if t==0,
    save A.mat A
    disp('Jacobian matrix saved as A in A.mat'),
end,

Pdot=A*P+P*A'+Q-P*C'*R1*C*P;
%Pdot=zeros(8,8);

sys = 3600*[xdot;SymReshape(Pdot);lambda*(1-x(21))];

% End of mdlDerivatives.
%
%==============================================================
% mdlOutputs
% Return the block outputs.
%==============================================================
%
function sys = mdlUpdate(t,x,u,dperiod,doffset)
global theta0 lambda
if (abs(round((t-doffset)/dperiod)-(t-doffset)/dperiod) < 1e-8) & (u(5)>0.5),
    load xini.mat
    x0=[x(1:5);xini(6:20);theta0]; % mode standard
%    x0=[xini;theta0]; % mode rinitialisation totale
%    x0=x(1:21); % mode calcul d'un tat statique
    sys = x0-x(1:21);
else
    sys = [];
end,

function sys = mdlOutputs(t,x,u)
Hrg = 1.53e5;
Prg = 254.4;
x=x(1:21)+x(22:42);
Tra = x(1);
Trg = x(2);
Crc = x(3);
Csc = x(4);
Fcf = x(5);
Rai = u(1);
Tra_mes = u(3);
Trg_mes = u(4);

Rcb=0;
Kod = 6.34e-9*Rai^2;
Ofg = 21-242*Rcb/Rai;
Kor=Ofg2Kor(Ofg,Rai,Crc);

InnovError=abs(Tra-Tra_mes)/Tra_mes+abs(Trg-Trg_mes)/Tra_mes;

%xn=[775;943;0.3;1.2;1e-4];
%Pn=xn*xn';
P=SymReshape(x(6:20));
%P=P./Pn;
%pmax=max(abs(SymReshape(P))); % following Krener's Idea

C=[eye(2),zeros(2,3)];
R1=eye(2);
K=P*C'*R1*[1;1];
pmax=norm(K);

theta=x(21);

%if t>23.8, % mode calcul d'un tat statique
%    save state.mat x
%    disp('Actual state saved in state.mat'),
%end,

sys = [Tra;Trg;Crc;Csc;Fcf;Rcb;Kor;InnovError;theta;pmax];
% End of mdlOutputs.